************************************************************************
**  ^ROUTINE: WTFQFR - Reverse communication level routine for Freund's
**                     transpose free QMR algorithm. (^)
**
      subroutine ztfqfr ( ijob , ireq , u , uexact , b , iva , ivql , 
     &   ivqr , iwk , fwk , iparm , fparm , ier )
**
**  ^DESCRIPTION:
**   Preconditioned transpose free QMR routine
**
**  ^AUTHOR:   wdj@beta.lanl.gov
**
**  ^MODIFIED: wdj@lithos.c3.lanl.gov Thu May  6 13:12:38 MDT 1993
**
**  ^ARGUMENTS: see *Subroutine Arguments* below.
**
**  ^REQUIREMENTS:
**    Common Blocks: none
**    Subroutines:   see below.
**
**  ^SIDE_EFFECTS:
**    See the definition of METHRDCLS to see which parameters
**    are modified
**
**  ^ALGORITHM:
**    TFQMR
**
**  ^REFERENCES:
**    R. W. Freund, ``A Transpose-Free Quasi-Minimal Residual
**    Algorithm for Non-Hermitian Linear Systems,'' {\sl SIAM
**    J. Sci. Stat. Comput.}
**
**  ^DOCUMENTATION:
**
************************************************************************
*
      implicit none
          Include 'fcube.h'
*         Include 'veclib.h'
*
*#    METHRAL        - (^)
      integer ijob
      integer ireq
      integer ier
       integer iwk(*)
       integer iparm(*)
       double complex fwk(*)
       double complex fparm(*)
       double complex u(*)
       double complex uexact(*)
       double complex b(*)
      integer ivql
      integer iva
      integer ivqr
*
        integer itimer
        double precision tc1, tr1, tc2, tr2
      integer iitab
      integer iftab
      integer iwkstg
      integer iclstg
      integer in
      integer is
      integer iretlb
      integer ivitmp, ivotmp
      integer iv1tst, iv2tst
      integer istab1, istab2, istab
      integer ib1,ib2, iu1,iu2
      integer iud
      parameter (iud =0)
      integer ibd
      parameter (ibd =1)
      integer ilbd
      parameter (ilbd =2)
      integer irlbd
      parameter (irlbd=3)
      integer ied
      parameter (ied =4)
      integer ird
      parameter (ird =5)
      integer ilrd
      parameter (ilrd =6)
      integer irlrd
      parameter (irlrd=7)
      double complex urndf
      double complex top
      double complex bot
      parameter (urndf=.1)
      double complex mydot1, mydot2
      external zargck
      external zckstg
      external zwrnd
      external zifppr
      external zitput
      external xtimer
      integer ipme
      integer iphost
      integer log2np
      integer nproc
      integer iom
      character*72 errstr
      external ximini
      external ximal
      external xifre
      external zfmini
      external zfmal
      external zffre
*     integer imv1, imv2        ! temporaries - for matvec
      integer irt
      integer ir
      integer iqlr
      integer iqrqlr
*
      integer iqlw
      integer iqrqlw
*
      integer iqlv
      integer iqrqlv
*
      integer iry
      integer iary
      integer ilary
      integer irlary
*
      integer irdd
      integer iard
      integer ilard
      integer irlard
*
      integer iv1
      integer iv2
      double complex rdot
      double complex rnorm
      double complex r0norm
      double complex tau
      double complex theta
      double complex eta
      double complex sigma
      double complex alpha
      double complex beta
      double complex c
      double complex rho
      double complex rhoold
      double complex uudot, uexdt
*
*
****^^******************************************************************
*     $Modified: spencer@navier.ae.utexas.edu on Thu May 16 11:59:19 1996 $
*     $Id: tfqfr.fm4,v 1.9 1994/11/22 05:20:46 joubert Exp $
*     $Revision: 1.9 $
*     $Log: tfqfr.fm4,v $
*     Revision 1.9  1994/11/22  05:20:46  joubert
*     misc changes for release 1.0
*
*     Revision 1.8  1994/09/28  19:25:13  joubert
*     mods to stats and ntest options
*
*     Revision 1.7  1994/07/06  05:52:53  joubert
*     mods to it meths
*
*     Revision 1.6  1994/07/05  22:21:04  joubert
*     lanczos/orthores acceleration; some cleanup of iterative methods
*
*     Revision 1.5  1994/06/24  18:21:12  joubert
*     t3d version mods; installed iom it meth
*
*     Revision 1.4  1994/06/07  17:40:29  joubert
*     installed orthores iterative method
*
*     Revision 1.3  1994/06/02  21:55:16  joubert
*     installed CM random vector routine
*
*     Revision 1.2  1994/04/30  04:41:59  joubert
*     improved error handling
*
*     Revision 1.1  1994/04/29  23:53:34  joubert
*     added tfqmr method
*
************************************************************************
*
      iitab = (iparm(6))
      iftab = (iparm(6))
      ir = (iparm(6))
      iqlr = (iparm(6))
      iqrqlr = (iparm(6))
      irt = (iparm(6))
      iqlw = (iparm(6))
      iqrqlw = (iparm(6))
      iqlv = (iparm(6))
      iqrqlv = (iparm(6))
      iry = (iparm(6))
      iary = (iparm(6))
      ilary = (iparm(6))
      irlary = (iparm(6))
      irdd = (iparm(6))
      iard = (iparm(6))
      ilard = (iparm(6))
      irlard = (iparm(6))
      iv1 = (iparm(6))
      iv2 = (iparm(6))
*
*
      if (ijob .ne. 1) go to 1000
*
      iclstg = 1
      itimer = 0
      in = 0
      iparm(5) = in
      is = 0
*
      iu1 = (iparm(6))
      iu2 = (iparm(6))
      ib1 = (iparm(6))
      ib2 = (iparm(6))
      istab1 = (iparm(6))
      istab2 = (iparm(6))
*
      ipme = mynode ( )
      iphost = myhost ( )
      log2np = nodedim ( )
      nproc = 2**log2np
      if (iparm(27) .eq. 0) iparm(27) = 0
      if (iparm(27) .eq. 0) then
        ier = 0
        if (ijob.eq.1 .or. ijob.eq.2 .or. ijob.eq.0) then
      call ximini ( iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmini ( iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        endif
        call zargck ( iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        call xtimer (tc1,tr1,iparm(22),1)
      endif
*
      call xtimer (tc1,tr1,iparm(22),0)
      itimer = 1
*
      if (iparm(27) .eq. 0) then
        call zifppr ( 1 , iparm, fparm, ier)
           if (ier .lt. 0) go to 900
      endif
*
      call ximal (iitab,33, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (iftab,14, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
*
*
      if (iparm(2) .ge. 3) then
        if (ipme .eq. 0) then
          write (iparm(1),'(/''     Transpose Free QMR Method (Freund Ve
     &rsion).''/)')
        endif
      endif
*
      call zfmal (irt,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (iqlr,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (iqlv,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (iqlw,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (irdd,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (iry,iparm(3)*(2), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (ilard,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (ilary,iparm(3)*(2), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      if (((iparm(13).eq.1).or.(iparm(13).eq.3))) then
        if (((mod(iparm(15)/2,2).eq.1) .or. iparm(12).eq.2 .or. iparm(
     &     12).eq.6)) then
      call zfmal (ir,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (iard,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (iary,iparm(3)*(2), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        endif
      else
        ir = iqlr
        iary = ilary
        iard = ilard
      endif
      if (((iparm(13).eq.2).or.(iparm(13).eq.3))) then
        if (((mod(iparm(15)/8,2).eq.1) .or. iparm(12).eq.4 .or. iparm(
     &     12).eq.8)) then
      call zfmal (iqrqlv,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (iqrqlw,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (iqrqlr,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (irlard,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (irlary,iparm(3)*(2), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        endif
      else
        iqrqlv = iqlv
        iqrqlw = iqlw
        iqrqlr = iqlr
        irlard = ilard
        irlary = ilary
      endif
      call zfmal (iv1,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (iv2,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
*
      if (iparm(14) .eq. -2 ) then
          call zwfil (u,(0d0,0.0d0), iparm , fparm , iwk , fwk , ier )
      endif
*
      if (iparm(14) .eq. 2 .or. iparm(14) .eq. -1) go to 5500
                 go to 5501
 5500  continue
          call zwrdot (mydot1,b, b, iparm , fparm , iwk , fwk , ier )
        call zwrnd (u, iparm , fparm , iwk , fwk , ier )
      call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          ivqr = ivitmp
          call zwset (fwk(ivqr),u, iparm , fparm , iwk , fwk , ier )
          iva = iv1
       ireq = 3
      iretlb = 2000
      go to 1500
 2000 continue
      call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
       if (ijob .eq. -1) go to 900
          call zwrdot (mydot2,fwk(iv1), fwk(iv1), iparm , fparm , iwk , 
     &       fwk , ier )
          call zwscl ( u, u, urndf*sqrt(mydot1/mydot2), iparm , fparm , 
     &       iwk , fwk , ier )
                go to 5502
 5501 continue
 5502 continue
*
      if (iparm(14) .eq. 3) go to 5503
                 go to 5504
 5503  continue
      call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          ivqr = ivitmp
          call zwset (fwk(ivqr),u, iparm , fparm , iwk , fwk , ier )
          iva = iv1
       ireq = 3
      iretlb = 2001
      go to 1500
 2001 continue
      call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
       if (ijob .eq. -1) go to 900
          call zwsub ( fwk(iv1), b, fwk(iv1), iparm , fparm , iwk , fwk 
     &       , ier )
          call zwrdot (mydot1,fwk(iv1), fwk(iv1), iparm , fparm , iwk , 
     &       fwk , ier )
          call zwset (fwk(iv1),u, iparm , fparm , iwk , fwk , ier )
        call zwrnd (u, iparm , fparm , iwk , fwk , ier )
      call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          ivqr = ivitmp
          call zwset (fwk(ivqr),u, iparm , fparm , iwk , fwk , ier )
          iva = iv2
       ireq = 3
      iretlb = 2002
      go to 1500
 2002 continue
      call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
       if (ijob .eq. -1) go to 900
          call zwrdot (mydot2,fwk(iv2), fwk(iv2), iparm , fparm , iwk , 
     &       fwk , ier )
          call zwsax (u,fwk(iv1), u, -urndf*sqrt(mydot1/mydot2), iparm ,
     &        fparm , iwk , fwk , ier )
                go to 5505
 5504 continue
 5505 continue
*
      if (iparm(21) .eq. 1) go to 5506
                 go to 5507
 5506  continue
      call zfmal (istab1,8, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (istab2,8, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (ib1,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (ib2,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (iu1,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (iu2,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        istab = istab1
        if (iparm(19) .eq. 1) then
          call zwdot (fwk(istab+iud), uexact, uexact, iparm , fparm , 
     &       iwk , fwk , ier )
        endif
          call zwdot (fwk(istab+ibd ), b, b, iparm , fparm , iwk , fwk ,
     &        ier )
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5509
                 go to 5510
 5509  continue
          call zwset (fwk(ib2),b, iparm , fparm , iwk , fwk , ier )
                go to 5511
 5510 continue
      call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          iva = ivitmp
          call zwset (fwk(iva),b, iparm , fparm , iwk , fwk , ier )
          ivql = ib2
       ireq = 5
      iretlb = 2003
      go to 1500
 2003 continue
      call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
       if (ijob .eq. -1) go to 900
 5511 continue
          call zwdot (fwk(istab+ilbd), fwk(ib2), fwk(ib2), iparm , 
     &       fparm , iwk , fwk , ier )
          call zwset (fwk(iu1),fwk(ib2), iparm , fparm , iwk , fwk , 
     &       ier )
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5512
                 go to 5513
 5512  continue
          call zwset (fwk(iu2),fwk(iu1), iparm , fparm , iwk , fwk , 
     &       ier )
                go to 5514
 5513 continue
          ivql = iu1
          ivqr = iu2
       ireq = 7
      iretlb = 2004
      go to 1500
 2004 continue
       if (ijob .eq. -1) go to 900
 5514 continue
          call zwdot (fwk(istab+irlbd), fwk(iu2), fwk(iu2), iparm , 
     &       fparm , iwk , fwk , ier )
        if (iparm(19) .eq. 1) then
          call zwsub ( fwk(iu1), u, uexact, iparm , fparm , iwk , fwk , 
     &       ier )
          call zwdot (fwk(istab+ied), fwk(iu1), fwk(iu1), iparm , fparm 
     &       , iwk , fwk , ier )
        endif
      call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          ivqr = ivitmp
          call zwset (fwk(ivqr),u, iparm , fparm , iwk , fwk , ier )
          iva = ib1
       ireq = 3
      iretlb = 2005
      go to 1500
 2005 continue
      call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
       if (ijob .eq. -1) go to 900
          call zwsub ( fwk(ib1), b, fwk(ib1), iparm , fparm , iwk , fwk 
     &       , ier )
          call zwdot (fwk(istab+ird), fwk(ib1), fwk(ib1), iparm , fparm 
     &       , iwk , fwk , ier )
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5515
                 go to 5516
 5515  continue
          call zwset (fwk(ib2),fwk(ib1), iparm , fparm , iwk , fwk , 
     &       ier )
                go to 5517
 5516 continue
          iva = ib1
          ivql = ib2
       ireq = 5
      iretlb = 2006
      go to 1500
 2006 continue
       if (ijob .eq. -1) go to 900
 5517 continue
          call zwdot (fwk(istab+ilrd ), fwk(ib2), fwk(ib2), iparm , 
     &       fparm , iwk , fwk , ier )
          call zwset (fwk(iu1),fwk(ib2), iparm , fparm , iwk , fwk , 
     &       ier )
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5518
                 go to 5519
 5518  continue
          call zwset (fwk(iu2),fwk(iu1), iparm , fparm , iwk , fwk , 
     &       ier )
                go to 5520
 5519 continue
          ivql = iu1
          ivqr = iu2
       ireq = 7
      iretlb = 2007
      go to 1500
 2007 continue
       if (ijob .eq. -1) go to 900
 5520 continue
          call zwdot (fwk(istab+irlrd), fwk(iu2), fwk(iu2), iparm , 
     &       fparm , iwk , fwk , ier )
        if (iparm(2) .ge. 4) then
        if (ipme .eq. 0) then
          write (iparm(1),'()')
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(1x, 60(''*''))')
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'()')
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''Initial iteration statistics'')')
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'()')
        endif
        if (iparm(19) .eq. 1) then
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of uexact            '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+iud))
        endif
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of b                 '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+ibd))
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of Ql*b              '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+ilbd))
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of Qr*Ql*b           '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+irlbd))
        endif
        if (iparm(19) .eq. 1) then
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of error             '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+ied))
        endif
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of residual          '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+ird))
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of Ql*r              '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+ilrd))
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of Qr*Ql*r           '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+irlrd))
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'()')
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(1x, 60(''*''))')
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'()')
        endif
        endif
      call zffre (iu2,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zffre (iu1,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zffre (ib2,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zffre (ib1,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
                go to 5508
 5507 continue
 5508 continue
*
      if (((iparm(14).eq.0).or.(iparm(14).eq.-2))) go to 5521
                 go to 5522
 5521  continue
        if (((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5524
                 go to 5525
 5524  continue
          if (((mod(iparm(15)/2,2).eq.1) .or. iparm(12).eq.2 .or. iparm(
     &       12).eq.6)) go to 5527
                 go to 5528
 5527  continue
          call zwset (fwk(ir),b, iparm , fparm , iwk , fwk , ier )
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5530
                 go to 5531
 5530  continue
          call zwset (fwk(iqlr),fwk(ir), iparm , fparm , iwk , fwk , 
     &       ier )
                go to 5532
 5531 continue
          iva = ir
          ivql = iqlr
       ireq = 5
      iretlb = 2008
      go to 1500
 2008 continue
       if (ijob .eq. -1) go to 900
 5532 continue
                go to 5529
 5528 continue
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5533
                 go to 5534
 5533  continue
          call zwset (fwk(iqlr),b, iparm , fparm , iwk , fwk , ier )
                go to 5535
 5534 continue
      call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          iva = ivitmp
          call zwset (fwk(iva),b, iparm , fparm , iwk , fwk , ier )
          ivql = iqlr
       ireq = 5
      iretlb = 2009
      go to 1500
 2009 continue
      call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
       if (ijob .eq. -1) go to 900
 5535 continue
 5529 continue
                go to 5526
 5525 continue
          call zwset (fwk(iqlr),b, iparm , fparm , iwk , fwk , ier )
 5526 continue
                go to 5523
 5522 continue
        if (((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5536
                 go to 5537
 5536  continue
          if (((mod(iparm(15)/2,2).eq.1) .or. iparm(12).eq.2 .or. iparm(
     &       12).eq.6)) go to 5539
                 go to 5540
 5539  continue
      call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          ivqr = ivitmp
          call zwset (fwk(ivqr),u, iparm , fparm , iwk , fwk , ier )
          iva = ir
       ireq = 3
      iretlb = 2010
      go to 1500
 2010 continue
      call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
       if (ijob .eq. -1) go to 900
          call zwsub ( fwk(ir), b, fwk(ir), iparm , fparm , iwk , fwk , 
     &       ier )
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5542
                 go to 5543
 5542  continue
          call zwset (fwk(iqlr),fwk(ir), iparm , fparm , iwk , fwk , 
     &       ier )
                go to 5544
 5543 continue
          iva = ir
          ivql = iqlr
       ireq = 5
      iretlb = 2011
      go to 1500
 2011 continue
       if (ijob .eq. -1) go to 900
 5544 continue
                go to 5541
 5540 continue
      call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          ivqr = ivitmp
          call zwset (fwk(ivqr),u, iparm , fparm , iwk , fwk , ier )
          iva = iv1
       ireq = 3
      iretlb = 2012
      go to 1500
 2012 continue
      call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
       if (ijob .eq. -1) go to 900
          call zwsub ( fwk(iv1), b, fwk(iv1), iparm , fparm , iwk , fwk 
     &       , ier )
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5545
                 go to 5546
 5545  continue
          call zwset (fwk(iqlr),fwk(iv1), iparm , fparm , iwk , fwk , 
     &       ier )
                go to 5547
 5546 continue
          iva = iv1
          ivql = iqlr
       ireq = 5
      iretlb = 2013
      go to 1500
 2013 continue
       if (ijob .eq. -1) go to 900
 5547 continue
 5541 continue
                go to 5538
 5537 continue
      call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          ivqr = ivitmp
          call zwset (fwk(ivqr),u, iparm , fparm , iwk , fwk , ier )
          iva = iqlr
       ireq = 3
      iretlb = 2014
      go to 1500
 2014 continue
      call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
       if (ijob .eq. -1) go to 900
          call zwsub ( fwk(iqlr), b, fwk(ir), iparm , fparm , iwk , fwk 
     &       , ier )
 5538 continue
 5523 continue
      if (((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5548
                 go to 5549
 5548  continue
        if (((mod(iparm(15)/8,2).eq.1) .or. iparm(12).eq.4 .or. iparm(
     &     12).eq.8)) go to 5551
                 go to 5552
 5551  continue
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5554
                 go to 5555
 5554  continue
          call zwset (fwk(iqrqlr),fwk(iqlr), iparm , fparm , iwk , fwk ,
     &        ier )
                go to 5556
 5555 continue
          ivql = iqlr
          ivqr = iqrqlr
       ireq = 7
      iretlb = 2015
      go to 1500
 2015 continue
       if (ijob .eq. -1) go to 900
 5556 continue
                go to 5553
 5552 continue
 5553 continue
                go to 5550
 5549 continue
 5550 continue
*
          call zwrdot (rdot,fwk(iqlr), fwk(iqlr), iparm , fparm , iwk , 
     &       fwk , ier )
      rnorm = sqrt(abs(rdot))
      r0norm = rnorm
*
*---------------------------iteration loop------------------------------
*
  100 continue
*
        if (iparm(2) .ge. 5) then
        if (ipme .eq. 0) then
          write (iparm(1),500) 'rnorm               ', rnorm
        endif
        endif
*
        if (iparm(12).eq.-3) go to 5557
                 go to 5558
 5557  continue
          if ((mod(iparm(15)/1,2).eq.1)) then
          endif
          if ((mod(iparm(15)/2,2).eq.1)) then
          iva = ir
          endif
          if ((mod(iparm(15)/4,2).eq.1)) then
          ivql = iqlr
          endif
          if ((mod(iparm(15)/8,2).eq.1)) then
          ivqr = iqrqlr
          endif
          ireq = 9
      iretlb = 2016
      go to 1500
 2016 continue
          if ((mod(iparm(15)/8,2).eq.1)) then
          endif
          if ((mod(iparm(15)/4,2).eq.1)) then
          endif
          if ((mod(iparm(15)/2,2).eq.1)) then
          endif
          if ((mod(iparm(15)/1,2).eq.1)) then
          endif
          if (ier .ge. 0) then
            call zitput ( in , is , iparm , fparm , iwk , fwk , ier )
          endif
          if (ijob .eq. -1) then
            if (iparm(2).ge.3 .and. iparm(27).eq.0) then
        if (ipme .eq. 0) then
          write (iparm(1),'(/a/)') ' User-requested termination of itera
     &tive method.'
        endif
            endif
            go to 900
          endif
                go to 5559
 5558 continue
 5559 continue
        if (iparm(12) .gt. 1) go to 5560
                 go to 5561
 5560  continue
          if (iparm(12) .eq. 1 ) then
            if (iparm(5) .eq. 0) then
              if (iparm(19) .eq. 0) then
                ier = -5
                call xersho ( ier, 'ztfqfr' , iparm , '19 and 12 incompa
     &tible' )
                go to 900
              endif
          call zwdot (bot, uexact, uexact, iparm , fparm , iwk , fwk , 
     &       ier )
              top = bot
            else
      call zfmal (iv1tst,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          call zwsub ( fwk(iv1tst), u, uexact, iparm , fparm , iwk , 
     &       fwk , ier )
          call zwdot (top, fwk(iv1tst), fwk(iv1tst), iparm , fparm , 
     &       iwk , fwk , ier )
      call zffre (iv1tst,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
            endif
          endif
          if (iparm(12) .eq. 2 ) then
            if (iparm(5) .eq. 0) then
          call zwdot (bot, b, b, iparm , fparm , iwk , fwk , ier )
              top = bot
            else
          call zwdot (top, fwk(ir), fwk(ir), iparm , fparm , iwk , fwk ,
     &        ier )
            endif
          endif
          if (iparm(12) .eq. 3) go to 5563
                 go to 5564
 5563  continue
            if (iparm(5) .eq. 0) go to 5566
                 go to 5567
 5566  continue
              if (((iparm(14).eq.0).or.(iparm(14).eq.-2))) go to 5569
                 go to 5570
 5569  continue
          call zwdot (bot, fwk(iqlr), fwk(iqlr), iparm , fparm , iwk , 
     &       fwk , ier )
                go to 5571
 5570 continue
      call zfmal (iv1tst,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5572
                 go to 5573
 5572  continue
          call zwset (fwk(iv1tst),b, iparm , fparm , iwk , fwk , ier )
                go to 5574
 5573 continue
      call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          iva = ivitmp
          call zwset (fwk(iva),b, iparm , fparm , iwk , fwk , ier )
          ivql = iv1tst
       ireq = 5
      iretlb = 2017
      go to 1500
 2017 continue
      call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
       if (ijob .eq. -1) go to 900
 5574 continue
          call zwdot (bot, fwk(iv1tst), fwk(iv1tst), iparm , fparm , 
     &       iwk , fwk , ier )
      call zffre (iv1tst,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
 5571 continue
              top = bot
                go to 5568
 5567 continue
          call zwdot (top, fwk(iqlr), fwk(iqlr), iparm , fparm , iwk , 
     &       fwk , ier )
 5568 continue
                go to 5565
 5564 continue
 5565 continue
          if (iparm(12) .eq. 4) go to 5575
                 go to 5576
 5575  continue
            if (iparm(5) .eq. 0) go to 5578
                 go to 5579
 5578  continue
              if (((iparm(14).eq.0).or.(iparm(14).eq.-2))) go to 5581
                 go to 5582
 5581  continue
          call zwdot (bot, fwk(iqrqlr), fwk(iqrqlr), iparm , fparm , 
     &       iwk , fwk , ier )
                go to 5583
 5582 continue
      call zfmal (iv1tst,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (iv2tst,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5584
                 go to 5585
 5584  continue
          call zwset (fwk(iv1tst),b, iparm , fparm , iwk , fwk , ier )
                go to 5586
 5585 continue
      call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          iva = ivitmp
          call zwset (fwk(iva),b, iparm , fparm , iwk , fwk , ier )
          ivql = iv1tst
       ireq = 5
      iretlb = 2018
      go to 1500
 2018 continue
      call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
       if (ijob .eq. -1) go to 900
 5586 continue
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5587
                 go to 5588
 5587  continue
          call zwset (fwk(iv2tst),fwk(iv1tst), iparm , fparm , iwk , 
     &       fwk , ier )
                go to 5589
 5588 continue
          ivql = iv1tst
          ivqr = iv2tst
       ireq = 7
      iretlb = 2019
      go to 1500
 2019 continue
       if (ijob .eq. -1) go to 900
 5589 continue
          call zwdot (bot, fwk(iv2tst), fwk(iv2tst), iparm , fparm , 
     &       iwk , fwk , ier )
      call zffre (iv2tst,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zffre (iv1tst,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
 5583 continue
              top = bot
                go to 5580
 5579 continue
          call zwdot (top, fwk(iqrqlr), fwk(iqrqlr), iparm , fparm , 
     &       iwk , fwk , ier )
 5580 continue
                go to 5577
 5576 continue
 5577 continue
          if (iparm(12) .eq. 1 ) then
            if (iparm(19) .eq. 0) then
                ier = -5
                call xersho ( ier, 'ztfqfr' , iparm , '19 and 12 incompa
     &tible' )
                go to 900
            endif
            if (((iparm(14).eq.0).or.(iparm(14).eq.-2))) then
          call zwdot (top, uexact, uexact, iparm , fparm , iwk , fwk , 
     &       ier )
            else
      call zfmal (iv1tst,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          call zwsub ( fwk(iv1tst), u, uexact, iparm , fparm , iwk , 
     &       fwk , ier )
          call zwdot (top, fwk(iv1tst), fwk(iv1tst), iparm , fparm , 
     &       iwk , fwk , ier )
      call zffre (iv1tst,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
            endif
            if (iparm(5) .eq. 0) bot = top
          endif
          if (iparm(12) .eq. 6) then
          call zwdot (top, fwk(ir), fwk(ir), iparm , fparm , iwk , fwk ,
     &        ier )
            if (iparm(5) .eq. 0) bot = top
          endif
          if (iparm(12) .eq. 7) then
          call zwdot (top, fwk(iqlr), fwk(iqlr), iparm , fparm , iwk , 
     &       fwk , ier )
            if (iparm(5) .eq. 0) bot = top
          endif
          if (iparm(12) .eq. 8) then
          call zwdot (top, fwk(iqrqlr), fwk(iqrqlr), iparm , fparm , 
     &       iwk , fwk , ier )
            if (iparm(5) .eq. 0) bot = top
          endif
          if (dble(top) .lt. dble((0d0,0.0d0))) then
                ier = -10
                call xersho ( ier, 'ztfqfr' , iparm , 'Attempt to take s
     &qrt of negative number' )
                go to 900
          endif
          if (dble(bot) .le. dble((0d0,0.0d0))) then
                ier = -10
                call xersho ( ier, 'ztfqfr' , iparm , 'Attempt to divide
     & by zero' )
                go to 900
          endif
          fparm(5) = sqrt(dble(top)/dble(bot))
          call zitput ( in , is , iparm , fparm , iwk , fwk , ier )
          if (dble(fparm(5)) .le. dble(fparm(4))) go to 2500
                go to 5562
 5561 continue
 5562 continue
*
        if (iparm(12) .eq. -1) then
          if (r0norm .eq. (0d0,0.0d0)) then
                ier = -10
                call xersho ( ier, 'ztfqfr' , iparm , 'Attempt to divide
     & by zero' )
                go to 900
          endif
          fparm(5) = rnorm/r0norm
          call zitput ( in , is , iparm , fparm , iwk , fwk , ier )
          if (dble(fparm(5)) .le. dble(fparm(4))) go to 2500
        endif
*
        if (iparm(12) .eq. -2) then
          call zwsub ( fwk(iv1), u, uexact, iparm , fparm , iwk , fwk , 
     &       ier )
          call zwrdot (uudot,fwk(iv1), fwk(iv1), iparm , fparm , iwk , 
     &       fwk , ier )
          call zwrdot (uexdt,uexact, uexact, iparm , fparm , iwk , fwk ,
     &        ier )
          if (dble(uudot) .lt. dble((0d0,0.0d0))) then
                ier = -10
                call xersho ( ier, 'ztfqfr' , iparm , 'Attempt to take s
     &qrt of negative number' )
                go to 900
          endif
          if (dble(uexdt) .le. dble((0d0,0.0d0))) then
                ier = -10
                call xersho ( ier, 'ztfqfr' , iparm , 'Attempt to divide
     & by zero' )
                go to 900
          endif
          fparm(5) = sqrt(dble(uudot)/dble(uexdt))
          call zitput ( in , is , iparm , fparm , iwk , fwk , ier )
          if (dble(fparm(5)) .le. dble(fparm(4))) go to 2500
        endif
*
        if (in .ge. iparm(4)) then
                ier = 2
                call xersho ( ier, 'ztfqfr' , iparm , ' ' )
          go to 900
        endif
*
        if (iparm(18) .ne. -1) then
          call zckstg ( iclstg, iwkstg, iparm , fparm , iwk , fwk , ier 
     &       )
           if (ier .lt. 0) go to 900
        endif
*
*-----------------------proceed with iteration--------------------------
*
        if (in .eq. 0) go to 5590
                 go to 5591
 5590  continue
          call zwset (fwk(irt ),fwk(iqlr), iparm , fparm , iwk , fwk , 
     &       ier )
          call zwset (fwk(iqlw),fwk(iqlr), iparm , fparm , iwk , fwk , 
     &       ier )
          if (((iparm(13).eq.2).or.(iparm(13).eq.3))) then
            if (((mod(iparm(15)/8,2).eq.1) .or. iparm(12).eq.4 .or. 
     &         iparm(12).eq.8)) then
          call zwset (fwk(iqrqlw),fwk(iqrqlr), iparm , fparm , iwk , 
     &       fwk , ier )
            endif
          endif
          if (((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5593
                 go to 5594
 5593  continue
            if (((mod(iparm(15)/8,2).eq.1) .or. iparm(12).eq.4 .or. 
     &         iparm(12).eq.8)) go to 5596
                 go to 5597
 5596  continue
          call zwset (fwk(iry+iparm(3)*((mod((in),2)))),fwk(iqrqlr), 
     &       iparm , fparm , iwk , fwk , ier )
                go to 5598
 5597 continue
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5599
                 go to 5600
 5599  continue
          call zwset (fwk(iry+iparm(3)*((mod((in),2)))),fwk(iqlr), 
     &       iparm , fparm , iwk , fwk , ier )
                go to 5601
 5600 continue
          ivql = iqlr
          ivqr = iry + iparm(3)*((mod((in),2)))
       ireq = 7
      iretlb = 2020
      go to 1500
 2020 continue
       if (ijob .eq. -1) go to 900
 5601 continue
 5598 continue
                go to 5595
 5594 continue
          call zwset (fwk(iry+iparm(3)*((mod((in),2)))),fwk(iqlr), 
     &       iparm , fparm , iwk , fwk , ier )
 5595 continue
          if (((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5602
                 go to 5603
 5602  continue
            if (((mod(iparm(15)/2,2).eq.1) .or. iparm(12).eq.2 .or. 
     &         iparm(12).eq.6)) go to 5605
                 go to 5606
 5605  continue
          ivqr = iry + iparm(3)*((mod((in),2)))
          iva = iary + iparm(3)*((mod((in),2)))
       ireq = 3
      iretlb = 2021
      go to 1500
 2021 continue
       if (ijob .eq. -1) go to 900
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5608
                 go to 5609
 5608  continue
          call zwset (fwk(ilary+iparm(3)*((mod((in),2)))),fwk(iary+
     &       iparm(3)*((mod((in),2)))), iparm , fparm , iwk , fwk , ier 
     &       )
                go to 5610
 5609 continue
          iva = iary + iparm(3)*((mod((in),2)))
          ivql = ilary + iparm(3)*((mod((in),2)))
       ireq = 5
      iretlb = 2022
      go to 1500
 2022 continue
       if (ijob .eq. -1) go to 900
 5610 continue
                go to 5607
 5606 continue
          ivqr = iry + iparm(3)*((mod((in),2)))
          iva = iv1
       ireq = 3
      iretlb = 2023
      go to 1500
 2023 continue
       if (ijob .eq. -1) go to 900
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5611
                 go to 5612
 5611  continue
          call zwset (fwk(ilary+iparm(3)*((mod((in),2)))),fwk(iv1), 
     &       iparm , fparm , iwk , fwk , ier )
                go to 5613
 5612 continue
          iva = iv1
          ivql = ilary + iparm(3)*((mod((in),2)))
       ireq = 5
      iretlb = 2024
      go to 1500
 2024 continue
       if (ijob .eq. -1) go to 900
 5613 continue
 5607 continue
                go to 5604
 5603 continue
          ivqr = iry + iparm(3)*((mod((in),2)))
          iva = ilary + iparm(3)*((mod((in),2)))
       ireq = 3
      iretlb = 2025
      go to 1500
 2025 continue
       if (ijob .eq. -1) go to 900
 5604 continue
          call zwset (fwk(iqlv),fwk(ilary+iparm(3)*((mod((in),2)))), 
     &       iparm , fparm , iwk , fwk , ier )
          if (((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5614
                 go to 5615
 5614  continue
            if (((mod(iparm(15)/8,2).eq.1) .or. iparm(12).eq.4 .or. 
     &         iparm(12).eq.8)) go to 5617
                 go to 5618
 5617  continue
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5620
                 go to 5621
 5620  continue
          call zwset (fwk(irlary+iparm(3)*((mod((in),2)))),fwk(ilary+
     &       iparm(3)*((mod((in),2)))), iparm , fparm , iwk , fwk , ier 
     &       )
                go to 5622
 5621 continue
          ivql = ilary + iparm(3)*((mod((in),2)))
          ivqr = irlary + iparm(3)*((mod((in),2)))
       ireq = 7
      iretlb = 2026
      go to 1500
 2026 continue
       if (ijob .eq. -1) go to 900
 5622 continue
          call zwset (fwk(iqrqlv),fwk(irlary+iparm(3)*((mod((in),2)))), 
     &       iparm , fparm , iwk , fwk , ier )
                go to 5619
 5618 continue
 5619 continue
                go to 5616
 5615 continue
 5616 continue
          tau = r0norm
          call zwdot (rho, fwk(irt), fwk(iqlr), iparm , fparm , iwk , 
     &       fwk , ier )
                go to 5592
 5591 continue
 5592 continue
*
        if (mod(in,2).eq.0 .and. in.ne.0) go to 5623
                 go to 5624
 5623  continue
          rhoold = rho
          call zwdot (rho, fwk(irt), fwk(iqlw), iparm , fparm , iwk , 
     &       fwk , ier )
          if (rhoold .eq. (0d0,0.0d0)) then
                ier = -7
                call xersho ( ier, 'ztfqfr' , iparm , ' ' )
                go to 900
          endif
          beta = rho / rhoold
          if (((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5626
                 go to 5627
 5626  continue
            if (((mod(iparm(15)/8,2).eq.1) .or. iparm(12).eq.4 .or. 
     &         iparm(12).eq.8)) go to 5629
                 go to 5630
 5629  continue
          call zwsax (fwk(iry+iparm(3)*((mod((in),2)))),fwk(iqrqlw), 
     &       fwk(iry+iparm(3)*((mod((in-1),2)))), beta, iparm , fparm , 
     &       iwk , fwk , ier )
                go to 5631
 5630 continue
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5632
                 go to 5633
 5632  continue
          call zwset (fwk(iv1),fwk(iqlw), iparm , fparm , iwk , fwk , 
     &       ier )
                go to 5634
 5633 continue
          ivql = iqlw
          ivqr = iv1
       ireq = 7
      iretlb = 2027
      go to 1500
 2027 continue
       if (ijob .eq. -1) go to 900
 5634 continue
          call zwsax (fwk(iry+iparm(3)*((mod((in),2)))),fwk(iv1), fwk(
     &       iry+iparm(3)*((mod((in-1),2)))), beta, iparm , fparm , iwk 
     &       , fwk , ier )
 5631 continue
                go to 5628
 5627 continue
          call zwsax (fwk(iry+iparm(3)*((mod((in),2)))),fwk(iqrqlw), 
     &       fwk(iry+iparm(3)*((mod((in-1),2)))), beta, iparm , fparm , 
     &       iwk , fwk , ier )
 5628 continue
          if (((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5635
                 go to 5636
 5635  continue
            if (((mod(iparm(15)/2,2).eq.1) .or. iparm(12).eq.2 .or. 
     &         iparm(12).eq.6)) go to 5638
                 go to 5639
 5638  continue
          ivqr = iry + iparm(3)*((mod((in),2)))
          iva = iary + iparm(3)*((mod((in),2)))
       ireq = 3
      iretlb = 2028
      go to 1500
 2028 continue
       if (ijob .eq. -1) go to 900
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5641
                 go to 5642
 5641  continue
          call zwset (fwk(ilary+iparm(3)*((mod((in),2)))),fwk(iary+
     &       iparm(3)*((mod((in),2)))), iparm , fparm , iwk , fwk , ier 
     &       )
                go to 5643
 5642 continue
          iva = iary + iparm(3)*((mod((in),2)))
          ivql = ilary + iparm(3)*((mod((in),2)))
       ireq = 5
      iretlb = 2029
      go to 1500
 2029 continue
       if (ijob .eq. -1) go to 900
 5643 continue
                go to 5640
 5639 continue
          ivqr = iry + iparm(3)*((mod((in),2)))
          iva = iv1
       ireq = 3
      iretlb = 2030
      go to 1500
 2030 continue
       if (ijob .eq. -1) go to 900
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5644
                 go to 5645
 5644  continue
          call zwset (fwk(ilary+iparm(3)*((mod((in),2)))),fwk(iv1), 
     &       iparm , fparm , iwk , fwk , ier )
                go to 5646
 5645 continue
          iva = iv1
          ivql = ilary + iparm(3)*((mod((in),2)))
       ireq = 5
      iretlb = 2031
      go to 1500
 2031 continue
       if (ijob .eq. -1) go to 900
 5646 continue
 5640 continue
                go to 5637
 5636 continue
          ivqr = iry + iparm(3)*((mod((in),2)))
          iva = ilary + iparm(3)*((mod((in),2)))
       ireq = 3
      iretlb = 2032
      go to 1500
 2032 continue
       if (ijob .eq. -1) go to 900
 5637 continue
          if (((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5647
                 go to 5648
 5647  continue
            if (((mod(iparm(15)/8,2).eq.1) .or. iparm(12).eq.4 .or. 
     &         iparm(12).eq.8)) go to 5650
                 go to 5651
 5650  continue
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5653
                 go to 5654
 5653  continue
          call zwset (fwk(irlary+iparm(3)*((mod((in),2)))),fwk(ilary+
     &       iparm(3)*((mod((in),2)))), iparm , fparm , iwk , fwk , ier 
     &       )
                go to 5655
 5654 continue
          ivql = ilary + iparm(3)*((mod((in),2)))
          ivqr = irlary + iparm(3)*((mod((in),2)))
       ireq = 7
      iretlb = 2033
      go to 1500
 2033 continue
       if (ijob .eq. -1) go to 900
 5655 continue
                go to 5652
 5651 continue
 5652 continue
                go to 5649
 5648 continue
 5649 continue
          call zwsax (fwk(iqlv),fwk(ilary+iparm(3)*((mod((in-1),2)))), 
     &       fwk(iqlv), beta, iparm , fparm , iwk , fwk , ier )
          call zwsax (fwk(iqlv),fwk(ilary+iparm(3)*((mod((in ),2)))), 
     &       fwk(iqlv), beta, iparm , fparm , iwk , fwk , ier )
          if (((iparm(13).eq.2).or.(iparm(13).eq.3))) then
            if (((mod(iparm(15)/8,2).eq.1) .or. iparm(12).eq.4 .or. 
     &         iparm(12).eq.8)) then
          call zwsax (fwk(iqrqlv),fwk(irlary+iparm(3)*((mod((in-1),2))))
     &       , fwk(iqrqlv), beta, iparm , fparm , iwk , fwk , ier )
          call zwsax (fwk(iqrqlv),fwk(irlary+iparm(3)*((mod((in ),2)))),
     &        fwk(iqrqlv), beta, iparm , fparm , iwk , fwk , ier )
            endif
          endif
                go to 5625
 5624 continue
 5625 continue
*
        if (mod(in,2) .eq. 0) go to 5656
                 go to 5657
 5656  continue
          call zwdot (sigma, fwk(irt), fwk(iqlv), iparm , fparm , iwk , 
     &       fwk , ier )
          if (sigma .eq. (0d0,0.0d0)) then
                ier = -7
                call xersho ( ier, 'ztfqfr' , iparm , ' ' )
                go to 900
          endif
          alpha = rho / sigma
          if (((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5659
                 go to 5660
 5659  continue
            if (((mod(iparm(15)/8,2).eq.1) .or. iparm(12).eq.4 .or. 
     &         iparm(12).eq.8)) go to 5662
                 go to 5663
 5662  continue
          call zwsax (fwk(iry+iparm(3)*((mod((in+1),2)))),fwk(iry+iparm(
     &       3)*((mod((in),2)))), fwk(iqrqlv), -alpha, iparm , fparm , 
     &       iwk , fwk , ier )
                go to 5664
 5663 continue
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5665
                 go to 5666
 5665  continue
          call zwset (fwk(iv1),fwk(iqlv), iparm , fparm , iwk , fwk , 
     &       ier )
                go to 5667
 5666 continue
          ivql = iqlv
          ivqr = iv1
       ireq = 7
      iretlb = 2034
      go to 1500
 2034 continue
       if (ijob .eq. -1) go to 900
 5667 continue
          call zwsax (fwk(iry+iparm(3)*((mod((in+1),2)))),fwk(iry+iparm(
     &       3)*((mod((in),2)))), fwk(iv1), -alpha, iparm , fparm , iwk 
     &       , fwk , ier )
 5664 continue
                go to 5661
 5660 continue
          call zwsax (fwk(iry+iparm(3)*((mod((in+1),2)))),fwk(iry+iparm(
     &       3)*((mod((in),2)))), fwk(iqrqlv), -alpha, iparm , fparm , 
     &       iwk , fwk , ier )
 5661 continue
          if (((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5668
                 go to 5669
 5668  continue
            if (((mod(iparm(15)/2,2).eq.1) .or. iparm(12).eq.2 .or. 
     &         iparm(12).eq.6)) go to 5671
                 go to 5672
 5671  continue
          ivqr = iry + iparm(3)*((mod((in+1),2)))
          iva = iary + iparm(3)*((mod((in+1),2)))
       ireq = 3
      iretlb = 2035
      go to 1500
 2035 continue
       if (ijob .eq. -1) go to 900
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5674
                 go to 5675
 5674  continue
          call zwset (fwk(ilary+iparm(3)*((mod((in+1),2)))),fwk(iary+
     &       iparm(3)*((mod((in+1),2)))), iparm , fparm , iwk , fwk , 
     &       ier )
                go to 5676
 5675 continue
          iva = iary + iparm(3)*((mod((in+1),2)))
          ivql = ilary + iparm(3)*((mod((in+1),2)))
       ireq = 5
      iretlb = 2036
      go to 1500
 2036 continue
       if (ijob .eq. -1) go to 900
 5676 continue
                go to 5673
 5672 continue
          ivqr = iry + iparm(3)*((mod((in+1),2)))
          iva = iv1
       ireq = 3
      iretlb = 2037
      go to 1500
 2037 continue
       if (ijob .eq. -1) go to 900
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5677
                 go to 5678
 5677  continue
          call zwset (fwk(ilary+iparm(3)*((mod((in+1),2)))),fwk(iv1), 
     &       iparm , fparm , iwk , fwk , ier )
                go to 5679
 5678 continue
          iva = iv1
          ivql = ilary + iparm(3)*((mod((in+1),2)))
       ireq = 5
      iretlb = 2038
      go to 1500
 2038 continue
       if (ijob .eq. -1) go to 900
 5679 continue
 5673 continue
                go to 5670
 5669 continue
          ivqr = iry + iparm(3)*((mod((in+1),2)))
          iva = ilary + iparm(3)*((mod((in+1),2)))
       ireq = 3
      iretlb = 2039
      go to 1500
 2039 continue
       if (ijob .eq. -1) go to 900
 5670 continue
          if (((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5680
                 go to 5681
 5680  continue
            if (((mod(iparm(15)/8,2).eq.1) .or. iparm(12).eq.4 .or. 
     &         iparm(12).eq.8)) go to 5683
                 go to 5684
 5683  continue
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5686
                 go to 5687
 5686  continue
          call zwset (fwk(irlary+iparm(3)*((mod((in+1),2)))),fwk(ilary+
     &       iparm(3)*((mod((in+1),2)))), iparm , fparm , iwk , fwk , 
     &       ier )
                go to 5688
 5687 continue
          ivql = ilary + iparm(3)*((mod((in+1),2)))
          ivqr = irlary + iparm(3)*((mod((in+1),2)))
       ireq = 7
      iretlb = 2040
      go to 1500
 2040 continue
       if (ijob .eq. -1) go to 900
 5688 continue
                go to 5685
 5684 continue
 5685 continue
                go to 5682
 5681 continue
 5682 continue
                go to 5658
 5657 continue
 5658 continue
*
          call zwsax (fwk(iqlw),fwk(iqlw), fwk(ilary+iparm(3)*((mod((in)
     &       ,2)))), -alpha, iparm , fparm , iwk , fwk , ier )
        if (((iparm(13).eq.2).or.(iparm(13).eq.3))) then
          if (((mod(iparm(15)/8,2).eq.1) .or. iparm(12).eq.4 .or. iparm(
     &       12).eq.8)) then
          call zwsax (fwk(iqrqlw),fwk(iqrqlw), fwk(irlary+iparm(3)*((
     &       mod((in),2)))), -alpha, iparm , fparm , iwk , fwk , ier )
          endif
        endif
        if (in .eq. 0) then
          call zwset (fwk(irdd),fwk(iry+iparm(3)*((mod((in),2)))), 
     &       iparm , fparm , iwk , fwk , ier )
          if (((iparm(13).eq.1).or.(iparm(13).eq.3))) then
            if (((mod(iparm(15)/2,2).eq.1) .or. iparm(12).eq.2 .or. 
     &         iparm(12).eq.6)) then
          call zwset (fwk(iard),fwk(iary+iparm(3)*((mod((in),2)))), 
     &       iparm , fparm , iwk , fwk , ier )
            endif
          endif
          call zwset (fwk(ilard),fwk(ilary+iparm(3)*((mod((in),2)))), 
     &       iparm , fparm , iwk , fwk , ier )
          if (((iparm(13).eq.2).or.(iparm(13).eq.3))) then
            if (((mod(iparm(15)/8,2).eq.1) .or. iparm(12).eq.4 .or. 
     &         iparm(12).eq.8)) then
          call zwset (fwk(irlard),fwk(irlary+iparm(3)*((mod((in),2)))), 
     &       iparm , fparm , iwk , fwk , ier )
            endif
          endif
        else
          if (alpha .eq. (0d0,0.0d0)) then
                ier = -7
                call xersho ( ier, 'ztfqfr' , iparm , ' ' )
                go to 900
          endif
          call zwsax (fwk(irdd),fwk(iry+iparm(3)*((mod((in),2)))), fwk(
     &       irdd), (theta**2*eta/alpha), iparm , fparm , iwk , fwk , 
     &       ier )
          if (((iparm(13).eq.1).or.(iparm(13).eq.3))) then
            if (((mod(iparm(15)/2,2).eq.1) .or. iparm(12).eq.2 .or. 
     &         iparm(12).eq.6)) then
          call zwsax (fwk(iard),fwk(iary+iparm(3)*((mod((in),2)))), fwk(
     &       iard), (theta**2*eta/alpha), iparm , fparm , iwk , fwk , 
     &       ier )
            endif
          endif
          call zwsax (fwk(ilard),fwk(ilary+iparm(3)*((mod((in),2)))), 
     &       fwk(ilard), (theta**2*eta/alpha), iparm , fparm , iwk , 
     &       fwk , ier )
          if (((iparm(13).eq.2).or.(iparm(13).eq.3))) then
            if (((mod(iparm(15)/8,2).eq.1) .or. iparm(12).eq.4 .or. 
     &         iparm(12).eq.8)) then
          call zwsax (fwk(irlard),fwk(irlary+iparm(3)*((mod((in),2)))), 
     &       fwk(irlard), (theta**2*eta/alpha), iparm , fparm , iwk , 
     &       fwk , ier )
            endif
          endif
        endif
          call zwrdot (theta,fwk(iqlw), fwk(iqlw), iparm , fparm , iwk ,
     &        fwk , ier )
        if (tau .eq. (0d0,0.0d0)) then
                ier = -7
                call xersho ( ier, 'ztfqfr' , iparm , ' ' )
                go to 900
        endif
        theta = sqrt(theta) / tau
        c = sqrt((1d0,0.0d0)+theta**2)
        if (c .eq. (0d0,0.0d0)) then
                ier = -7
                call xersho ( ier, 'ztfqfr' , iparm , ' ' )
                go to 900
        endif
        c = (1d0,0.0d0)/c
        tau = tau*theta*c
        eta = c**2 * alpha
          call zwsax (u,u, fwk(irdd), eta, iparm , fparm , iwk , fwk , 
     &       ier )
        if (((iparm(13).eq.1).or.(iparm(13).eq.3))) then
          if (((mod(iparm(15)/2,2).eq.1) .or. iparm(12).eq.2 .or. iparm(
     &       12).eq.6)) then
          call zwsax (fwk(ir),fwk(ir), fwk(iard), -eta, iparm , fparm , 
     &       iwk , fwk , ier )
          endif
        endif
          call zwsax (fwk(iqlr),fwk(iqlr), fwk(ilard), -eta, iparm , 
     &       fparm , iwk , fwk , ier )
        if (((iparm(13).eq.2).or.(iparm(13).eq.3))) then
          if (((mod(iparm(15)/8,2).eq.1) .or. iparm(12).eq.4 .or. iparm(
     &       12).eq.8)) then
          call zwsax (fwk(iqrqlr),fwk(iqrqlr), fwk(irlard), -eta, iparm 
     &       , fparm , iwk , fwk , ier )
          endif
        endif
*
          call zwrdot (rdot,fwk(iqlr), fwk(iqlr), iparm , fparm , iwk , 
     &       fwk , ier )
        rnorm = sqrt(abs(rdot))
*
        in = in + 1
        iparm(5) = in
        is = is + 1
*
      go to 100
*
 2500 continue
      if (iparm(2) .ge. 3) then
        if (ipme .eq. 0) then
          write (iparm(1),'(/a,i7,a/)') '     Transpose Free QMR Method 
     &has converged in ', iparm(5), ' iterations.'
        endif
      endif
*
*-------------------------------terminate-------------------------------
*
  900 continue
*
      if (iclstg .ne. 1) then
        if (iparm(18) .ne. -1) then
          call zckstg (-1, iwkstg, iparm , fparm , iwk , fwk , ier )
        endif
      endif
*
      if (iparm(21).eq.1 .and. ier.ge.0) go to 5689
                 go to 5690
 5689  continue
      call zfmal (ib1,iparm(3), iparm , fparm , iwk , fwk , ier )
      call zfmal (ib2,iparm(3), iparm , fparm , iwk , fwk , ier )
      call zfmal (iu1,iparm(3), iparm , fparm , iwk , fwk , ier )
      call zfmal (iu2,iparm(3), iparm , fparm , iwk , fwk , ier )
        istab = istab2
        if (iparm(19) .eq. 1) then
          call zwdot (fwk(istab+iud), uexact, uexact, iparm , fparm , 
     &       iwk , fwk , ier )
        endif
          call zwdot (fwk(istab+ibd ), b, b, iparm , fparm , iwk , fwk ,
     &        ier )
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5692
                 go to 5693
 5692  continue
          call zwset (fwk(ib2),b, iparm , fparm , iwk , fwk , ier )
                go to 5694
 5693 continue
      call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
          iva = ivitmp
          call zwset (fwk(iva),b, iparm , fparm , iwk , fwk , ier )
          ivql = ib2
       ireq = 5
      iretlb = 2041
      go to 1500
 2041 continue
      call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
       if (ijob .eq. -1) go to 900
 5694 continue
          call zwdot (fwk(istab+ilbd), fwk(ib2), fwk(ib2), iparm , 
     &       fparm , iwk , fwk , ier )
          call zwset (fwk(iu1),fwk(ib2), iparm , fparm , iwk , fwk , 
     &       ier )
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5695
                 go to 5696
 5695  continue
          call zwset (fwk(iu2),fwk(iu1), iparm , fparm , iwk , fwk , 
     &       ier )
                go to 5697
 5696 continue
          ivql = iu1
          ivqr = iu2
       ireq = 7
      iretlb = 2042
      go to 1500
 2042 continue
       if (ijob .eq. -1) go to 900
 5697 continue
          call zwdot (fwk(istab+irlbd), fwk(iu2), fwk(iu2), iparm , 
     &       fparm , iwk , fwk , ier )
        if (iparm(19) .eq. 1) then
          call zwsub ( fwk(iu1), u, uexact, iparm , fparm , iwk , fwk , 
     &       ier )
          call zwdot (fwk(istab+ied), fwk(iu1), fwk(iu1), iparm , fparm 
     &       , iwk , fwk , ier )
        endif
      call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
          ivqr = ivitmp
          call zwset (fwk(ivqr),u, iparm , fparm , iwk , fwk , ier )
          iva = ib1
       ireq = 3
      iretlb = 2043
      go to 1500
 2043 continue
      call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
       if (ijob .eq. -1) go to 900
          call zwsub ( fwk(ib1), b, fwk(ib1), iparm , fparm , iwk , fwk 
     &       , ier )
          call zwdot (fwk(istab+ird), fwk(ib1), fwk(ib1), iparm , fparm 
     &       , iwk , fwk , ier )
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5698
                 go to 5699
 5698  continue
          call zwset (fwk(ib2),fwk(ib1), iparm , fparm , iwk , fwk , 
     &       ier )
                go to 5700
 5699 continue
          iva = ib1
          ivql = ib2
       ireq = 5
      iretlb = 2044
      go to 1500
 2044 continue
       if (ijob .eq. -1) go to 900
 5700 continue
          call zwdot (fwk(istab+ilrd ), fwk(ib2), fwk(ib2), iparm , 
     &       fparm , iwk , fwk , ier )
          call zwset (fwk(iu1),fwk(ib2), iparm , fparm , iwk , fwk , 
     &       ier )
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5701
                 go to 5702
 5701  continue
          call zwset (fwk(iu2),fwk(iu1), iparm , fparm , iwk , fwk , 
     &       ier )
                go to 5703
 5702 continue
          ivql = iu1
          ivqr = iu2
       ireq = 7
      iretlb = 2045
      go to 1500
 2045 continue
       if (ijob .eq. -1) go to 900
 5703 continue
          call zwdot (fwk(istab+irlrd), fwk(iu2), fwk(iu2), iparm , 
     &       fparm , iwk , fwk , ier )
        if (iparm(19) .eq. 1) then
          if (fwk(istab1+ied) .eq. (0d0,0.0d0)) then
                ier = 4
                call xersho ( ier, 'ztfqfr' , iparm , 'Value of RELERR m
     &ay be inaccurate' )
          else
            fparm(8) = sqrt(fwk(istab2+ied)/fwk(istab1+ied))
          endif
        endif
        if (fwk(istab1+ird) .eq. (0d0,0.0d0)) then
                ier = 4
                call xersho ( ier, 'ztfqfr' , iparm , 'Value of RELRSD m
     &ay be inaccurate' )
        else
          fparm(7) = sqrt(fwk(istab2+ird)/fwk(istab1+ird))
        endif
        if (iparm(2) .ge. 4) then
        if (ipme .eq. 0) then
          write (iparm(1),'()')
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(1x, 60(''*''))')
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'()')
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''Final iteration statistics'')')
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'()')
        endif
        if (iparm(19) .eq. 1) then
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of uexact            '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+iud))
        endif
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of b                 '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+ibd))
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of Ql*b              '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+ilbd))
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of Qr*Ql*b           '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+irlbd))
        endif
        if (iparm(19) .eq. 1) then
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of error             '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+ied))
        endif
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of residual          '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+ird))
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of Ql*r              '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+ilrd))
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''2-norm of Qr*Ql*r           '',1pg20.10
     &,2x,1pg20.10)') sqrt(fwk(istab+irlrd))
        endif
        if (iparm(19) .eq. 1) then
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''Relative 2-norm of error    '',1pg20.10
     &,2x,1pg20.10)') fparm(8)
        endif
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(5x,''Relative 2-norm of residual '',1pg20.10
     &,2x,1pg20.10)') fparm(7)
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'()')
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'(1x, 60(''*''))')
        endif
        if (ipme .eq. 0) then
          write (iparm(1),'()')
        endif
        endif
      call zffre (iu2,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
      call zffre (iu1,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
      call zffre (ib2,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
      call zffre (ib1,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
      call zffre (istab2,8, iparm , fparm , iwk , fwk , ier )
      call zffre (istab1,8, iparm , fparm , iwk , fwk , ier )
                go to 5691
 5690 continue
 5691 continue
*
      call zffre (iv2 ,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
      call zffre (iv1 ,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
      if (((iparm(13).eq.2).or.(iparm(13).eq.3))) then
        if (((mod(iparm(15)/8,2).eq.1) .or. iparm(12).eq.4 .or. iparm(
     &     12).eq.8)) then
      call zffre (irlary,iparm(3)*(2), iparm , fparm , iwk , fwk , ier )
      call zffre (irlard,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
      call zffre (iqrqlr,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
      call zffre (iqrqlw,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
      call zffre (iqrqlv,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
        endif
      endif
      if (((iparm(13).eq.1).or.(iparm(13).eq.3))) then
        if (((mod(iparm(15)/2,2).eq.1) .or. iparm(12).eq.2 .or. iparm(
     &     12).eq.6)) then
      call zffre (iary,iparm(3)*(2), iparm , fparm , iwk , fwk , ier )
      call zffre (iard,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
      call zffre (ir,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
        endif
      endif
      call zffre (ilary,iparm(3)*(2), iparm , fparm , iwk , fwk , ier )
      call zffre (ilard,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
      call zffre (iry,iparm(3)*(2), iparm , fparm , iwk , fwk , ier )
      call zffre (irdd,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
      call zffre (iqlw,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
      call zffre (iqlv,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
      call zffre (iqlr,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
      call zffre (irt,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
*
*
*
*
      call zffre (iftab,14, iparm , fparm , iwk , fwk , ier )
      call xifre (iitab,33, iparm , fparm , iwk , fwk , ier )
*
      if (itimer .eq. 1) then
        call xtimer (tc2,tr2,iparm(22),0)
        fparm(1) = fparm(1) + (tc2-tc1)
        fparm(2) = fparm(2) + (tr2-tr1)
      endif
*
  910 continue
      if (iparm(27) .eq. 0) then
        call zifppr ( -1 , iparm, fparm, ier )
      endif
*
      if (iparm(27) .eq. 0) iparm(27) = 0
      iparm(11) = (iparm(6))
      ireq = -1
      return
*
*-------------------------------brancher--------------------------------
*
*
 1000 continue
*
      ipme = mynode ( )
      iphost = myhost ( )
      log2np = nodedim ( )
      nproc = 2**log2np
      iitab = iparm(11)
      if (iitab.eq.(iparm(6)) .and. ijob.eq.-1) go to 910
*
      if (iparm(11) .eq. (iparm(6))) then
                ier = -4
                call xersho ( ier, 'ztfqfr' , iparm , 'Values of ijob an
     &d iparm(iptr) incompatible' )
      endif
*
      call xtimer (tc1,tr1,iparm(22),0)
*
        iitab = iwk(iitab+0)
        iftab = iwk(iitab+1)
        istab = iwk(iitab+2)
        iwkstg = iwk(iitab+3)
        iclstg = iwk(iitab+4)
        in = iwk(iitab+5)
        is = iwk(iitab+6)
        itimer = iwk(iitab+7)
        iretlb = iwk(iitab+8)
        ivitmp = iwk(iitab+9)
        ivotmp = iwk(iitab+10)
        iv1tst = iwk(iitab+11)
        iv2tst = iwk(iitab+12)
        istab1 = iwk(iitab+13)
        istab2 = iwk(iitab+14)
        ir = iwk(iitab+15)
        iqlr = iwk(iitab+16)
        iqrqlr = iwk(iitab+17)
        irt = iwk(iitab+18)
        iqlw = iwk(iitab+19)
        iqrqlw = iwk(iitab+20)
        iqlv = iwk(iitab+21)
        iqrqlv = iwk(iitab+22)
        iry = iwk(iitab+23)
        iary = iwk(iitab+24)
        ilary = iwk(iitab+25)
        irlary = iwk(iitab+26)
        irdd = iwk(iitab+27)
        iard = iwk(iitab+28)
        ilard = iwk(iitab+29)
        irlard = iwk(iitab+30)
        iv1 = iwk(iitab+31)
        iv2 = iwk(iitab+32)
        top = fwk(iftab+0)
        bot = fwk(iftab+1)
        rdot = fwk(iftab+2)
        rnorm = fwk(iftab+3)
        r0norm = fwk(iftab+4)
        tau = fwk(iftab+5)
        theta = fwk(iftab+6)
        eta = fwk(iftab+7)
        sigma = fwk(iftab+8)
        alpha = fwk(iftab+9)
        beta = fwk(iftab+10)
        c = fwk(iftab+11)
        rho = fwk(iftab+12)
        rhoold = fwk(iftab+13)
*
      if (ijob.ne.3 .and. ijob.ne.-1) then
                ier = -4
                call xersho ( ier, 'ztfqfr' , iparm , 'Argument ijob inv
     &alid' )
      endif
*
      go to (2000,2001,2002,2003,2004,2005,2006,2007,2008,2009,2010,
     &   2011,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022,
     &   2023,2024,2025,2026,2027,2028,2029,2030,2031,2032,2033,2034,
     &   2035,2036,2037,2038,2039,2040,2041,2042,2043,2044,2045), 
     &   iretlb - 2000 + 1
*
*---------------------------request of caller---------------------------
*
*
 1500 continue
      iparm(11) = iitab
        iwk(iitab+0) = iitab
        iwk(iitab+1) = iftab
        iwk(iitab+2) = istab
        iwk(iitab+3) = iwkstg
        iwk(iitab+4) = iclstg
        iwk(iitab+5) = in
        iwk(iitab+6) = is
        iwk(iitab+7) = itimer
        iwk(iitab+8) = iretlb
        iwk(iitab+9) = ivitmp
        iwk(iitab+10) = ivotmp
        iwk(iitab+11) = iv1tst
        iwk(iitab+12) = iv2tst
        iwk(iitab+13) = istab1
        iwk(iitab+14) = istab2
        iwk(iitab+15) = ir
        iwk(iitab+16) = iqlr
        iwk(iitab+17) = iqrqlr
        iwk(iitab+18) = irt
        iwk(iitab+19) = iqlw
        iwk(iitab+20) = iqrqlw
        iwk(iitab+21) = iqlv
        iwk(iitab+22) = iqrqlv
        iwk(iitab+23) = iry
        iwk(iitab+24) = iary
        iwk(iitab+25) = ilary
        iwk(iitab+26) = irlary
        iwk(iitab+27) = irdd
        iwk(iitab+28) = iard
        iwk(iitab+29) = ilard
        iwk(iitab+30) = irlard
        iwk(iitab+31) = iv1
        iwk(iitab+32) = iv2
        fwk(iftab+0) = top
        fwk(iftab+1) = bot
        fwk(iftab+2) = rdot
        fwk(iftab+3) = rnorm
        fwk(iftab+4) = r0norm
        fwk(iftab+5) = tau
        fwk(iftab+6) = theta
        fwk(iftab+7) = eta
        fwk(iftab+8) = sigma
        fwk(iftab+9) = alpha
        fwk(iftab+10) = beta
        fwk(iftab+11) = c
        fwk(iftab+12) = rho
        fwk(iftab+13) = rhoold
      call xtimer (tc2,tr2,iparm(22),0)
      fparm(1) = fparm(1) + (tc2-tc1)
      fparm(2) = fparm(2) + (tr2-tr1)
      return
*
*
*--------------------------------end------------------------------------
*
  500 format ( 1x, a, 1pg20.10,2x,1pg20.10 )
*
      end
